/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.quickview;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.Icon;

import ca.uvic.csr.shrimp.LayoutConstants;
import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.QueryView.QueryViewAction;
import ca.uvic.csr.shrimp.resource.ResourceHandler;
import ca.uvic.csr.shrimp.usercontrols.ActionStartingListener;
import ca.uvic.csr.shrimp.usercontrols.DefaultProjectAction;
import ca.uvic.csr.shrimp.usercontrols.DefaultViewAction;
import ca.uvic.csr.shrimp.usercontrols.NestedTreemapViewAction;


/**
 * Main class for nested and flat quick views.
 * Most of the quick view configuration is done inside the {@link ExpressViewConfigurator} class.
 * This class has many static methods for creating different kinds of quick views - nested, flat, composite etc.
 * 
 * @see DefaultViewAction
 * @see NestedTreemapViewAction
 * @see QueryViewAction
 * @author Chris Callendar
 * @date 1-Aug-06
 */
public class QuickViewAction extends DefaultProjectAction {
	
	protected ExpressViewConfigurator config;
	private DataBean dataBean;
	private String iconFilename;
	protected Vector listeners = new Vector(1);

	/**
	 * Don't use this constructor.  It is used to instantiate this class using reflection.
	 */
	public QuickViewAction() {
		this(null, null, null);
	}
		
	public QuickViewAction(String actionName, ShrimpProject project) {
		this(actionName, (String)null, project);
	}

	public QuickViewAction(String actionName, String iconFilename, ShrimpProject project) {
		super(actionName, project);
		setIconFilename(iconFilename);	// also updates the icon inside a try/catch
		config = createExpressViewConfigurator(project);
	}
	
	public void dispose() {
		listeners = new Vector(1);
		if (config != null) {
			config.dispose();
			config = null;
		}
		dataBean = null;
	}

	protected ExpressViewConfigurator createExpressViewConfigurator(ShrimpProject project) {
		return new ExpressViewConfigurator(project);
	}
	
	public void setProject(ShrimpProject project) {
		config.setProject(project);
		super.setProject(project);
	}
	
	protected DataBean getDataBean() {
	    if (dataBean == null && (getProject() != null)) {
		    try {
	            dataBean = (DataBean) getProject().getBean(ShrimpProject.DATA_BEAN);
	        } catch (BeanNotFoundException e) {
	            e.printStackTrace();
	        }
	    }
        return dataBean;
	}

	/**
	 * Sets up the {@link ExpressViewConfigurator} and calls {@link ExpressViewConfigurator#configureView(String)}.
	 * Nothing will happen if the data bean or project are null.
	 * Before the action starts all {@link ActionStartingListener}s are notified.
	 * @see QuickViewAction#addActionStartedListener(ActionStartingListener)
	 */
	public void startAction() {
	    if (!hasProject()) {
	        return;
	    }
	    fireActionStartingEvent();
	    config.configureView(getActionName());
	}
	
	/** Notifies listeners that the action is about to start */
	private void fireActionStartingEvent() {
		if (listeners.size() > 0) {
			Vector clones = (Vector) listeners.clone();
			for (Iterator iter = clones.iterator(); iter.hasNext(); ) {
				ActionStartingListener listener = (ActionStartingListener) iter.next();
				listener.actionStarting();
			}
		}
	}

	public void addActionStartedListener(ActionStartingListener listener) {
		if (!listeners.contains(listener)) {
			listeners.add(listener);
		}
	}
	
	public void removeActionStartedListener(ActionStartingListener listener) {
		listeners.remove(listener);
	}
	
	public ExpressViewConfigurator getConfigurator() {
		return config;
	}
		
	public Icon getIcon() {
		return (Icon) getValue(SMALL_ICON);
	}
	
	public String getIconFilename() {
		return iconFilename;
	}
	
	/**
	 * Sets the icon filename.  If the filename is not null, then the {@link ResourceHandler} 
	 * is used to update the icon.
	 * @param iconFilename
	 */
	public void setIconFilename(String iconFilename) {
		this.iconFilename = iconFilename;
		if (iconFilename != null) {
			try {
				putValue(SMALL_ICON, ResourceHandler.getIcon(iconFilename));
			} catch (Exception ex) {}
		}
	}
	
	/**
	 * Checks if any cprels are defined, if so true is returned.
	 * @return true if the quick view is nested. 
	 */
	public boolean isNested() {
		return (config.getCprels().length > 0);
	}

	/**
	 * @return true if the view uses composite arcs
	 */
	public boolean isComposite() {
		return (config.getCompositeArcs().size() > 0);
	}

	/**
	 * Gets all the artifact types from the databean.
	 */
	public Collection getAllNodeTypes() {
		Collection nodes = Collections.EMPTY_LIST;
		if (getDataBean() != null) {
			nodes = getDataBean().getArtifactTypes(true, true);
		}
		return nodes;
	}

	/**
	 * Gets all the relationship types from the databean.
	 */
	public Collection getAllArcTypes() {
		Collection arcs = Collections.EMPTY_LIST;
		if (getDataBean() != null) {
			arcs = getDataBean().getRelationshipTypes(true, true);
		}
		return arcs;
	}
	
	/**
	 * Returns the default cprels.
	 */
	public String[] getDefaultCprels() {
		String[] defCprels = new String[0];
		if (getDataBean() != null) {
			defCprels = getDataBean().getDefaultCprels(); 
		}
		return defCprels;
	}
	
	/**
	 * Gets if the default cprels are inverted.
	 */
	public boolean getDefaultCprelsInverted() {
		boolean defInverted = false;
		if (getDataBean() != null) {
			defInverted = getDataBean().getDefaultCprelsInverted();
		}
		return defInverted;
	}
	
	/**
	 * Checks if the chosen node and arc types of interest are found in the 
	 * collection of all node and arc types for this project.
	 * For nested actions at least one node type of interest must be present.  For flat
	 * actions at least one node and one arc type must be chosen.
	 * @return boolean true if valid
	 */
	public boolean isValid() {
		return (isNested() ? isValidNodes() : isValidNodes() && isValidArcs());
	}
	
	private boolean isValidNodes() {
		// check that at least one node type is specified and is in the databean
		Collection nodes = config.getNodeTypesOfInterest();
		if (nodes.size() > 0) {
			Collection allNodes = getAllNodeTypes();
			if (allNodes.size() > 0) {
				// iterate through all the node types of interest until we find one node type 
				// that is in the collection of all node types for this project
				for (Iterator iter = nodes.iterator(); iter.hasNext(); ) {
					Object nodeType = iter.next();
					if (allNodes.contains(nodeType)) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	private boolean isValidArcs() {
		// check that at least one arc type is specified and is in the databean
		Collection arcs = config.getArcTypesOfInterest();
		if (arcs.size() > 0) {
			Collection allArcs = getAllArcTypes();
			if (allArcs.size() > 0) {
				// iterate through all the arc types of interest until we find one arc type 
				// that is in the collection of all arc types for this project
				for (Iterator iter = arcs.iterator(); iter.hasNext(); ) {
					Object arcType = iter.next();
					if (allArcs.contains(arcType)) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	
	// Static methods for creating specific quick views

	/** 
	 * Creates a nested quick view with all nodes and arcs being of interest.
	 * Uses a layout of {@link LayoutConstants#LAYOUT_GRID_BY_TYPE} and a label mode of {@link DisplayConstants#LABEL_MODE_FIXED}.
	 * Also uses default cprels and default inverted.
	 */
	public static QuickViewAction createNestedView(String actionName, ShrimpProject project) {
		QuickViewAction action = createNestedView(actionName, project, Collections.EMPTY_LIST, Collections.EMPTY_LIST, LayoutConstants.LAYOUT_GRID_BY_TYPE, 
				DisplayConstants.LABEL_MODE_FIXED);
		action.config.setNodeTypesOfInterest(action.getAllNodeTypes());
		action.config.setArcTypesOfInterest(action.getAllArcTypes());
		return action;
	}	
	
	/** 
	 * Creates a nested quick view with the given node and arc types of interest.
	 * Uses a layout of {@link LayoutConstants#LAYOUT_GRID_BY_TYPE} and a label mode of {@link DisplayConstants#LABEL_MODE_FIXED}.
	 * Also uses default cprels and default inverted.
	 */
	public static QuickViewAction createNestedView(ShrimpProject project, Collection nodeTypes, Collection arcTypes) {
		return createNestedView("Nested View", project, nodeTypes, arcTypes, LayoutConstants.LAYOUT_GRID_BY_TYPE, 
				DisplayConstants.LABEL_MODE_FIXED);
	}

	/** 
	 * Creates a nested quick view with the given node and arc types of interest and sets the layout and label modes.
	 * Uses default cprels and default inverted.
	 */
	public static QuickViewAction createNestedView(String actionName, ShrimpProject project, Collection nodeTypes, 
			Collection arcTypes, String layoutMode, String labelMode) {
		QuickViewAction action = createNestedView(actionName, project, nodeTypes, arcTypes, LayoutConstants.LAYOUT_GRID_BY_TYPE, 
				DisplayConstants.LABEL_MODE_FIXED, new String[0], false);
		action.config.setCprels(action.getDefaultCprels());
		action.config.setInverted(action.getDefaultCprelsInverted());
		return action;
	}
	
	/** 
	 * Creates a nested quick view with the given node and arc types of interest.
	 * Also sets the layout and label modes and cprels.
	 */
	public static QuickViewAction createNestedView(String actionName, ShrimpProject project, Collection nodeTypes, 
			Collection arcTypes, String layoutMode, String labelMode, String[] cprels, boolean inverted) {
		QuickViewAction action = new QuickViewAction(actionName, project);
		action.config.setNodeTypesOfInterest(nodeTypes);
		action.config.setArcTypesOfInterest(arcTypes);
		action.config.setLayoutMode(layoutMode);
		action.config.setLabelMode(labelMode);
		action.config.setCprels(cprels);
		action.config.setInverted(inverted);
		return action;
	}

	/**
	 * Creates a nested composite quick view.
	 * Layout used is a {@link LayoutConstants#LAYOUT_SPRING} and the Label mode is {@link DisplayConstants#LABEL_MODE_FIXED}.
	 * Default cprels are used.
	 */
	public static QuickViewAction createNestedCompositeView(String actionName, ShrimpProject project, Collection nodeTypes, 
			Collection arcTypes, Collection nodeTypesToOpen, Map compositeArcsMap) {
		QuickViewAction action = createNestedCompositeView(actionName, project, nodeTypes, arcTypes, 
				LayoutConstants.LAYOUT_SPRING, DisplayConstants.LABEL_MODE_FIXED, new String[0], false, 
				nodeTypesToOpen, compositeArcsMap);
		action.config.setCprels(action.getDefaultCprels());
		action.config.setInverted(action.getDefaultCprelsInverted());
		return action;
	}
	
	/**
	 * Creates a nested composite quick view.
	 */
	public static QuickViewAction createNestedCompositeView(String actionName, ShrimpProject project, Collection nodeTypes, 
			Collection arcTypes, String layoutMode, String labelMode, String[] cprels, boolean inverted, 
			Collection nodeTypesToOpen, Map compositeArcsMap) {
		return createGenericView(actionName, project, nodeTypes, arcTypes, layoutMode, labelMode, cprels, 
				inverted, nodeTypesToOpen, compositeArcsMap);
	}	
	
	
	/**
	 * Creates a flat quick view with all node and arc types being of interest.  
	 * A layout of {@value LayoutConstants#LAYOUT_TREE_VERTICAL} is used and a 
	 * label mode of {@link DisplayConstants#LABEL_MODE_FIXED} is used.
	 * There are no cprels or composite arcs since it is a flat graph.
	 */
	public static QuickViewAction createFlatView(String actionName, ShrimpProject project) {
		QuickViewAction action = createFlatView(actionName, project, Collections.EMPTY_LIST, Collections.EMPTY_LIST, 
				LayoutConstants.LAYOUT_TREE_VERTICAL, DisplayConstants.LABEL_MODE_FIXED);
		action.config.setNodeTypesOfInterest(action.getAllNodeTypes());
		action.config.setArcTypesOfInterest(action.getAllArcTypes());
		return action;
	}	
	/**
	 * Creates a flat quick view.  
	 * A layout of {@value LayoutConstants#LAYOUT_TREE_VERTICAL} is used and a 
	 * label mode of {@link DisplayConstants#LABEL_MODE_FIXED} is used.
	 * There are no cprels or composite arcs since it is a flat graph.
	 */
	public static QuickViewAction createFlatView(String actionName, ShrimpProject project, Collection nodeTypesOfInterest, 
			Collection arcTypesOfInterest) {
		return createFlatView(actionName, project, nodeTypesOfInterest, arcTypesOfInterest, LayoutConstants.LAYOUT_TREE_VERTICAL, 
				DisplayConstants.LABEL_MODE_FIXED);
	}
	
	/**
	 * Creates a flat quick view.  
	 * There are no cprels or composite arcs since it is a flat graph.
	 */
	public static QuickViewAction createFlatView(String actionName, ShrimpProject project, Collection nodeTypesOfInterest, 
			Collection arcTypesOfInterest, String layoutMode, String labelMode) {	
		QuickViewAction action = new QuickViewAction(actionName, project);
	    action.config.setNodeTypesOfInterest(nodeTypesOfInterest);
	    action.config.setArcTypesOfInterest(arcTypesOfInterest);
	    action.config.setLayoutMode(layoutMode);
	    action.config.setLabelMode(labelMode);
	    action.config.setCprels(new String[0]);
	    action.config.setInverted(false);
		return action;
	}

	/**
	 * Creates a generic QuickView and sets all the properties of the {@link ExpressViewConfigurator}.
	 */
	public static QuickViewAction createGenericView(String actionName, ShrimpProject project, Collection nodeTypes, 
			Collection arcTypes, String layoutMode, String labelMode, String[] cprels, boolean inverted, 
			Collection nodeTypesToOpen, Map compositeArcsMap) {
		QuickViewAction action = new QuickViewAction(actionName, project);
		action.config.setNodeTypesOfInterest(nodeTypes);
		action.config.setArcTypesOfInterest(arcTypes);
		action.config.setLayoutMode(layoutMode);
		action.config.setLabelMode(labelMode);
		action.config.setCprels(cprels);
		action.config.setInverted(inverted);
		action.config.setNodeTypesToOpen(nodeTypesToOpen);
		action.config.setCompositeArcs(compositeArcsMap);		
		return action;
	}
	
}
